(* -----------------------------------------------------------------------------
  RGB Color Channels      : 1.0
  Copyright  1986-2014   : Adirondack Software & Graphics
  Last Modification       : 03-09-2014
  Compiler                : Delphi XE4
  Target Platform         : Win32/Win64
  ImageEnVersion          : 5.0.6
  Operating System        : Windows 8.1
  Description             : Unit1.pas Unit
  Functions:              : Seperate RGB colors channels into layers
  : Change luminosity (brightness) of selected
  color channel
  : Combine color channels into RGB Composite image
  : Save the image with a new filename
  Notes                   : When the color channels are separated into red,
  green and blue with ImageEn, the images are grayscale, but they can be
  adjusted to show the color represented by the layer by processing each color
  channel layer with an ielOR operation, then by subtracting the other two color
  channels with IntensityRGBAll.  Notice that after you get the RGB channels,
  the image looks just like a regular composite RGB image, except each layer
  holds a red, green or blue color channel.  This is because when the layers are
  aligned (all the pixels are aligned), all three channels are used to display
  the pixels, so even though there is a Blue, Green and Red layer, the image
  appears as a composite RGB image on the screen.  This is similar to how a
  CRT monitor or TV displays full color images by using each of its red, blue
  and green electron guns. If a layer is selected and moved around you can see
  how the image changes.  Finally, when the channels are combined, the layers
  are positioned over each other (Realigned) then the layers are merged back
  into one bitmap. If you want to realign the layers back to position 0, 0
  without combining the color channels, select the Align button.

  This file is copyright  W W Miller, 1986-2014.
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  This project requires Delphi 2010 or higher to compile without error.
  --------------------------------------------------------------------------- *)
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ExtDlgs, ComCtrls,
  imageenview, ieview, imageenproc, imageenio, hyiedefs, hyieutils, iemview;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Open1: TButton;
    GetRGBChannels1: TButton;
    CombineChannels1: TButton;
    OpenPictureDialog1: TOpenPictureDialog;
    Splitter1: TSplitter;
    ImageEnView1: TImageEnView;
    AlignLayers1: TButton;
    Clear1: TButton;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    Panel3: TPanel;
    SaveAs1: TButton;
    SavePictureDialog1: TSavePictureDialog;
    Exit1: TButton;
    Panel4: TPanel;
    ImageEnMView1: TImageEnMView;
    ImageEnViewOriginal1: TImageEnView;
    HeaderControl1: THeaderControl;
    Panel5: TPanel;
    Label2: TLabel;
    Brightness1: TTrackBar;
    Reload1: TButton;
    Panel6: TPanel;
    Preview1: TImageEnView;
    HeaderControl2: THeaderControl;
    Reset1: TButton;
    HeaderControl3: THeaderControl;
    BrightnessLabel1: TLabel;
    Splitter2: TSplitter;
    Exit2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure GetRGBChannels1Click(Sender: TObject);
    procedure CombineChannels1Click(Sender: TObject);
    procedure AlignLayers1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure ImageEnView1LayerNotify(Sender: TObject; layer: Integer;
      event: TIELayerEvent);
    procedure SaveAs1Click(Sender: TObject);
    procedure SavePictureDialog1TypeChange(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
    procedure Brightness1Change(Sender: TObject);
    procedure Reload1Click(Sender: TObject);
    procedure Reset1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    AFilename: string;
    ieBitmapRGB: TIEBitmap;
    procedure ClearStatusbar;
    procedure UpdateGUI;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.UpdateGUI;
{ Update Toolbar buttons and components }
begin
  Open1.Enabled := True;
  Reload1.Enabled := FileExists(AFilename);
  GetRGBChannels1.Enabled := (FileExists(AFilename)) and
    (ImageEnView1.LayersCount = 1);
  AlignLayers1.Enabled := ImageEnView1.LayersCount = 3;
  CombineChannels1.Enabled := ImageEnView1.LayersCount = 3;
  Clear1.Enabled := FileExists(AFilename);
  SaveAs1.Enabled := ImageEnView1.LayersCount = 1;
  Brightness1.Enabled := ImageEnView1.LayersCount = 3;
  Reset1.Enabled := Brightness1.Position <> 0;
end;

procedure TForm1.AlignLayers1Click(Sender: TObject);
{ Align all the layers to position 0. }
var
  i: Integer;
begin
  if ImageEnView1.LayersCount = 3 then
  begin
    for i := 0 to ImageEnView1.LayersCount - 1 do
    begin
      ImageEnView1.Layers[i].PosX := 0;
      ImageEnView1.Layers[i].PosY := 0;
    end;
    ImageEnView1.Update;
    ImageEnView1.LayersDrawTo(Preview1.IEBitmap);
    Preview1.Update;
  end;
  UpdateGUI;
end;

procedure TForm1.ClearStatusbar;
{ ClearStatusbar. }
var
  i: Integer;
begin
  for i := 0 to StatusBar1.Panels.Count - 1 do
    StatusBar1.Panels[i].Text := '';
end;

procedure TForm1.SaveAs1Click(Sender: TObject);
{ Save the image with a new filename. }
begin
  SavePictureDialog1.FileName := ExtractFileName(AFilename);
  if SavePictureDialog1.Execute then
  begin
    Screen.Cursor := crHourglass;
    try
      ImageEnView1.IO.SaveToFile(SavePictureDialog1.FileName);
      Caption := 'RGB Channels - ' + AFilename;
      StatusBar1.Panels[0].Text := ExtractFileDir(AFilename);
      StatusBar1.Panels[1].Text := ExtractFileName(AFilename);
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  UpdateGUI;
end;

procedure TForm1.SavePictureDialog1TypeChange(Sender: TObject);
{ Change the File Extension when the FileType changes. }
var
  FilePath: string;
  FileName: string;
  FileExt: string;
begin
  FilePath := SavePictureDialog1.FileName;
  FileExt := ExtractFileExt(FilePath);
  FileName := ExtractFileName(FilePath);
end;

procedure TForm1.Brightness1Change(Sender: TObject);
{ Increase or decrease luminosity or brightness of selected color channel. }
begin
  Screen.Cursor := crHourglass;
  try
    BrightnessLabel1.Caption := IntToStr(Brightness1.Position);
    ImageEnView1.IEBitmap.ChannelOffset[0] := Brightness1.Position;
    ImageEnView1.IEBitmap.ChannelOffset[1] := Brightness1.Position;
    ImageEnView1.IEBitmap.ChannelOffset[2] := Brightness1.Position;
    ImageEnView1.Update;
    ImageEnView1.LayersDrawTo(Preview1.IEBitmap);
    Preview1.Update;
    UpdateGUI;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.Clear1Click(Sender: TObject);
{ Clear all layers and thumbnails. }
begin
  ImageEnMView1.Clear;
  ImageEnView1.LayersClear;
  ImageEnView1.Clear;
  ImageEnViewOriginal1.Clear;
  Preview1.Clear;
  Caption := 'RGB Channels';
  ClearStatusbar;
  UpdateGUI;
end;

procedure TForm1.CombineChannels1Click(Sender: TObject);
{ Combine the layers to recreate the RGB image. }
var
  i: Integer;
begin
  if ImageEnView1.LayersCount = 3 then
  begin
    Screen.Cursor := crHourglass;
    try
      { Make sure the layers are aligned }
      for i := 0 to ImageEnView1.LayersCount - 1 do
      begin
        ImageEnView1.Layers[i].PosX := 0;
        ImageEnView1.Layers[i].PosY := 0;
      end;
      ImageEnView1.Update;

      { Merge all the layers to recreate the RGB full color bitmap }
      ImageEnView1.LayersMergeAll;

      { Clear ImageEnMView }
      ImageEnMView1.Clear;
      { Add the recreated rgb full color bitmap to ImageEnMView }
      // iImage := ImageEnMView1.AppendImage;
      // ImageEnMView1.SetIEBitmap(iImage, ImageEnView1.IEBitmap);
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  UpdateGUI;
end;

procedure TForm1.Exit1Click(Sender: TObject);
{ Exit to windows. }
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
{ Initialize. }
begin
  { Register picture formats }
  IERegisterFormats;
  { Set the dialog filters }
  OpenPictureDialog1.Filter := GraphicFilter(TGraphic);
  SavePictureDialog1.Filter := GraphicFilter(TGraphic);
  { Allow the user to reposition the layers }
  ImageEnView1.Gestures.LayerMove.Enabled := True;
  ImageEnView1.MouseInteract := [miMoveLayers];
  ImageEnViewOriginal1.Blank;
  Preview1.Blank;
  { Use modern styling for ImageEnMView1 }
  ImageEnMView1.SetModernStyling();
  { Create an ieBitmap to hold the composite RGB image }
  ieBitmapRGB := TIEBitmap.Create;
  UpdateGUI;
end;

procedure TForm1.FormDestroy(Sender: TObject);
{ Free the ieRGBBitmap. }
begin
  IEUnregisterFormats;
  ieBitmapRGB.Free;
end;

procedure TForm1.FormShow(Sender: TObject);
var
  iImageEnIO: TImageEnIO;
begin
  AFilename := 'girl.jpg';
  iImageEnIO := TImageEnIO.Create(nil);
  try
    iImageEnIO.AttachedIEBitmap := ieBitmapRGB;
    iImageEnIO.LoadFromFile(AFilename);
    ImageEnViewOriginal1.IEBitmap.Assign(ieBitmapRGB);
    ImageEnViewOriginal1.Update;
    Preview1.IEBitmap.Assign(ieBitmapRGB);
    Preview1.Update;
    UpdateGUI;
    Caption := 'RGB Channels - ' + AFilename;
    StatusBar1.Panels[0].Text := ExtractFileDir(AFilename);
    StatusBar1.Panels[1].Text := ExtractFileName(AFilename);
  finally
    iImageEnIO.Free;
  end;
end;

procedure TForm1.GetRGBChannels1Click(Sender: TObject);
{ Get the image's red, green and blue channels and add them to a layer
  and add to ImageEnMView. }
var
  iLayer: Integer;
  iImage: Integer;
  ieBitmapRed: TIEBitmap;
  ieBitmapGreen: TIEBitmap;
  ieBitmapBlue: TIEBitmap;
begin
  if ImageEnView1.LayersCount = 1 then
  begin
    Screen.Cursor := crHourglass;
    try
      { Create a TIEBitmap to hold the red channel }
      ieBitmapRed := TIEBitmap.Create;
      try
        { Create a TIEBitmap to hold the green channel }
        ieBitmapGreen := TIEBitmap.Create;
        try
          { Create a TIEBitmap to hold the blue channel }
          ieBitmapBlue := TIEBitmap.Create;
          try
            { ieBitmapRGB has the color image }
            ImageEnView1.Proc.AttachedIEBitmap := ieBitmapRGB;
            { Get the Blue channel }
            ieBitmapBlue := ImageEnView1.Proc.GetRGBChannel(0);
            { Get the Green channel }
            ieBitmapGreen := ImageEnView1.Proc.GetRGBChannel(1);
            { Get the Red channel }
            ieBitmapRed := ImageEnView1.Proc.GetRGBChannel(2);

            { Layer 0 has the blue channel ieBitmap }
            iLayer := 0;
            { Give the layer a name }
            ImageEnView1.Layers[iLayer].Name := 'Blue Channel';
            ImageEnView1.Proc.AttachedIEBitmap := ieBitmapBlue;
            { Process the layer with a ielNormal operation }
            ImageEnView1.Layers[iLayer].Operation := ielNormal;
            { Subtract the red and green channels leaving the blue channel }
            ImageEnView1.Proc.IntensityRGBall(-255, -255, 0);
            { Assign the blue bitmap to layer 0 }
            ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapBlue);
            { Add a blue bitmap to ImageEnMView }
            iImage := ImageEnMView1.AppendImage;
            ImageEnMView1.SetIEBitmap(iImage, ieBitmapBlue);

            { Layer 1 has the Green channel ieBitmap }
            iLayer := ImageEnView1.LayersAdd;
            { Give the layer a name }
            ImageEnView1.Layers[iLayer].Name := 'Green Channel';
            ImageEnView1.Proc.AttachedIEBitmap := ieBitmapGreen;
            { Process the layer with a ielOR operation }
            ImageEnView1.Layers[iLayer].Operation := ielOR;
            { Subtract the red and blue channels leaving the green channel }
            ImageEnView1.Proc.IntensityRGBall(-255, 0, -255);
            { Assign the green bitmap to layer 0 }
            ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapGreen);
            { Add a green bitmap to ImageEnMView }
            iImage := ImageEnMView1.AppendImage;
            ImageEnMView1.SetIEBitmap(iImage, ieBitmapGreen);

            { Layer 2 has Red channel ieBitmap }
            iLayer := ImageEnView1.LayersAdd;
            { Give the layer a name }
            ImageEnView1.Layers[iLayer].Name := 'Red Channel';
            ImageEnView1.Proc.AttachedIEBitmap := ieBitmapRed;
            { Process the layer with a ielOR operation }
            ImageEnView1.Layers[iLayer].Operation := ielOR;
            { Subtract the green and blue channels leaving the red channel }
            ImageEnView1.Proc.IntensityRGBall(0, -255, -255);
            { Assign the red bitmap to layer 0 }
            ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapRed);
            { Add a red bitmap to ImageEnMView }
            iImage := ImageEnMView1.AppendImage;
            ImageEnMView1.SetIEBitmap(iImage, ieBitmapRed);
            ImageEnView1.LayersDrawTo(Preview1.IEBitmap);
            Preview1.Update;
            { Select the first image }
            ImageEnMView1.SelectedImage := 0;
          finally
            ieBitmapBlue.Free;
          end;
        finally
          ieBitmapGreen.Free;
        end;
      finally
        ieBitmapRed.Free;
      end;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  UpdateGUI;
end;

procedure TForm1.ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
{ Select corresponding layer. }
begin
  Brightness1.Position := 0;
  ImageEnView1.LayersCurrent := idx;
  StatusBar1.Panels[2].Text := string(ImageEnView1.Layers[idx].Name);
end;

procedure TForm1.ImageEnView1LayerNotify(Sender: TObject; layer: Integer;
  event: TIELayerEvent);
{ Select corresponding thumbnail. }
begin
  if event = ielSelected then
  begin
    ImageEnMView1.SelectedImage := layer;
    StatusBar1.Panels[2].Text := string(ImageEnView1.Layers[layer].Name);
  end;
end;

procedure TForm1.Open1Click(Sender: TObject);
{ Open an image and assign it to a TIEBitmap, and add the image as a thumbnail. }
var
  iImageEnIO: TImageEnIO;
begin
  if OpenPictureDialog1.Execute then
  begin
    Screen.Cursor := crHourglass;
    try
      AFilename := OpenPictureDialog1.FileName;
      iImageEnIO := TImageEnIO.Create(nil);
      try
        iImageEnIO.AttachedIEBitmap := ieBitmapRGB;
        iImageEnIO.LoadFromFile(AFilename);
        ImageEnViewOriginal1.IEBitmap.Assign(ieBitmapRGB);
        ImageEnViewOriginal1.Update;
        Preview1.IEBitmap.Assign(ieBitmapRGB);
        Preview1.Update;
      finally
        iImageEnIO.Free;
      end;
      UpdateGUI;
      Caption := 'RGB Color Channels - ' + AFilename;
      StatusBar1.Panels[0].Text := ExtractFileDir(AFilename);
      StatusBar1.Panels[1].Text := ExtractFileName(AFilename);
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TForm1.Reload1Click(Sender: TObject);
{ Reload. }
var
  iImageEnIO: TImageEnIO;
begin
  if FileExists(AFilename) then
  begin
    Screen.Cursor := crHourglass;
    try
      iImageEnIO := TImageEnIO.Create(nil);
      try
        iImageEnIO.AttachedIEBitmap := ieBitmapRGB;
        iImageEnIO.LoadFromFile(AFilename);
        ImageEnViewOriginal1.IEBitmap.Assign(ieBitmapRGB);
        ImageEnViewOriginal1.Update;
        Caption := 'RGB Color Channels - ' + AFilename;
        StatusBar1.Panels[0].Text := ExtractFileDir(AFilename);
        StatusBar1.Panels[1].Text := ExtractFileName(AFilename);
      finally
        iImageEnIO.Free;
      end;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TForm1.Reset1Click(Sender: TObject);
{ Reset. }
var
  iLayer: Integer;
  iImage: Integer;
  ieBitmapRed: TIEBitmap;
  ieBitmapGreen: TIEBitmap;
  ieBitmapBlue: TIEBitmap;
begin
  Brightness1.Position := 0;
  BrightnessLabel1.Caption := IntToStr(Brightness1.Position);
  ImageEnView1.Clear;
  ImageEnView1.LayersClear;
  ImageEnMView1.Clear;
  { Create a TIEBitmap to hold the red channel }
  ieBitmapRed := TIEBitmap.Create;
  try
    { Create a TIEBitmap to hold the green channel }
    ieBitmapGreen := TIEBitmap.Create;
    try
      { Create a TIEBitmap to hold the blue channel }
      ieBitmapBlue := TIEBitmap.Create;
      try
        { ieBitmapRGB has the color image }
        ImageEnView1.Proc.AttachedIEBitmap := ieBitmapRGB;
        { Get the Blue channel }
        ieBitmapBlue := ImageEnView1.Proc.GetRGBChannel(0);
        { Get the Green channel }
        ieBitmapGreen := ImageEnView1.Proc.GetRGBChannel(1);
        { Get the Red channel }
        ieBitmapRed := ImageEnView1.Proc.GetRGBChannel(2);

        { Layer 0 has the blue channel ieBitmap }
        iLayer := 0;
        { Give the layer a name }
        ImageEnView1.Layers[iLayer].Name := 'Blue Channel';
        ImageEnView1.Proc.AttachedIEBitmap := ieBitmapBlue;
        { Process the layer with a ielNormal operation }
        ImageEnView1.Layers[iLayer].Operation := ielNormal;
        { Subtract the red and green channels leaving the blue channel }
        ImageEnView1.Proc.IntensityRGBall(-255, -255, 0);
        { Assign the blue bitmap to layer 0 }
        ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapBlue);
        { Add a blue bitmap to ImageEnMView }
        iImage := ImageEnMView1.AppendImage;
        ImageEnMView1.SetIEBitmap(iImage, ieBitmapBlue);

        { Layer 1 has the Green channel ieBitmap }
        iLayer := ImageEnView1.LayersAdd;
        { Give the layer a name }
        ImageEnView1.Layers[iLayer].Name := 'Green Channel';
        ImageEnView1.Proc.AttachedIEBitmap := ieBitmapGreen;
        { Process the layer with a ielOR operation }
        ImageEnView1.Layers[iLayer].Operation := ielOR;
        { Subtract the red and blue channels leaving the green channel }
        ImageEnView1.Proc.IntensityRGBall(-255, 0, -255);
        { Assign the green bitmap to layer 0 }
        ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapGreen);
        { Add a green bitmap to ImageEnMView }
        iImage := ImageEnMView1.AppendImage;
        ImageEnMView1.SetIEBitmap(iImage, ieBitmapGreen);

        { Layer 2 has Red channel ieBitmap }
        iLayer := ImageEnView1.LayersAdd;
        { Give the layer a name }
        ImageEnView1.Layers[iLayer].Name := 'Red Channel';
        ImageEnView1.Proc.AttachedIEBitmap := ieBitmapRed;
        { Process the layer with a ielOR operation }
        ImageEnView1.Layers[iLayer].Operation := ielOR;
        { Subtract the green and blue channels leaving the red channel }
        ImageEnView1.Proc.IntensityRGBall(0, -255, -255);
        { Assign the red bitmap to layer 0 }
        ImageEnView1.Layers[iLayer].Bitmap.Assign(ieBitmapRed);
        { Add a red bitmap to ImageEnMView }
        iImage := ImageEnMView1.AppendImage;
        ImageEnMView1.SetIEBitmap(iImage, ieBitmapRed);
        ImageEnView1.LayersDrawTo(Preview1.IEBitmap);
        Preview1.Update;
        { Select the first image }
        ImageEnMView1.SelectedImage := 0;

      finally
        ieBitmapBlue.Free;
      end;
    finally
      ieBitmapGreen.Free;
    end;
  finally
    ieBitmapRed.Free;
  end;

  Brightness1.Position := 0;
  Preview1.IEBitmap.Assign(ieBitmapRGB);
  Preview1.Update;
end;

end.
